@@ -23,6 +23,7 @@
 #include <config.h>
 #include <libnotify/notify.h>
 #include <libgnome/gnome-i18n.h>
+#include <string.h>
 #include "xchat-plugin.h"
 #include "xg-plugin.h"
 
@@ -66,7 +67,7 @@ add_notify (char *summary, char *message)
 
 	notify = notify_notification_new (summary, message, NULL, NULL);
 	notify_notification_set_urgency (notify, NOTIFY_URGENCY_NORMAL);
-	notify_notification_set_icon_data_from_pixbuf (notify, notify_icon);
+	notify_notification_set_icon_from_pixbuf (notify, notify_icon);
 	if (!notify_notification_show (notify, &error)) {
 		g_warning (_("Failed to send notification: %s\n"), error->message);
 		g_error_free (error);
@@ -97,7 +98,7 @@ static int
 new_msg_cb (char *word[], gpointer data)
 {
 	const char *channel;
-	gchar *stripped, *message, *summary;
+	gchar *stripped, *escaped, *message, *summary;
 
 	if (focused)
 		return XCHAT_EAT_NONE;
@@ -106,16 +107,19 @@ new_msg_cb (char *word[], gpointer data)
 	stripped = xchat_strip (ph, word[2], -1, STRIP_COLORS | STRIP_ATTRS);
 
 	message = g_strdup_printf ("<%s> %s", word[1], stripped);
+	escaped = g_markup_escape_text(message, strlen(message));
+
 	if (channel[0] == '#')
 		summary = g_strdup_printf (_("Message in %s"), channel);
 	else
 		summary = g_strdup_printf (_("Message from %s"), channel);
 
-	add_notify (summary, message);
+	add_notify (summary, escaped);
 
 	xchat_free (ph, stripped);
 	g_free (message);
 	g_free (summary);
+	g_free (escaped);
 	return XCHAT_EAT_NONE;
 }
 
