@@ -16,6 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -60,24 +61,20 @@ thread_mbox (char *str)
 #endif
 
 /* leave this before XSUB.h, to avoid readdir() being redefined */
+
 static void
-perl_auto_load (void)
+perl_auto_load_from_path (const char *path)
 {
 	DIR *dir;
 	struct dirent *ent;
-	const char *xdir;
 
-	/* get the dir in local filesystem encoding (what opendir() expects!) */
-	xdir = xchat_get_info (ph, "xchatdirfs");
-	if (!xdir)						  /* xchatdirfs is new for 2.0.9, will fail on older */
-		xdir = xchat_get_info (ph, "xchatdir");
-	dir = opendir (xdir);
+	dir = opendir (path);
 	if (dir) {
 		while ((ent = readdir (dir))) {
 			int len = strlen (ent->d_name);
 			if (len > 3 && strcasecmp (".pl", ent->d_name + len - 3) == 0) {
-				char *file = malloc (len + strlen (xdir) + 2);
-				sprintf (file, "%s/%s", xdir, ent->d_name);
+				char *file = malloc (len + strlen (path) + 2);
+				sprintf (file, "%s/%s", path, ent->d_name);
 				perl_load_file (file);
 				free (file);
 			}
@@ -86,6 +83,26 @@ perl_auto_load (void)
 	}
 }
 
+static void
+perl_auto_load (void)
+{
+	const char *xdir;
+
+	/* get the dir in local filesystem encoding (what opendir() expects!) */
+	xdir = xchat_get_info (ph, "xchatdirfs");
+	if (!xdir)						  /* xchatdirfs is new for 2.0.9, will fail on older */
+		xdir = xchat_get_info (ph, "xchatdir");
+
+	/* autoload from ~/.xchat2/ or ${APPDATA}\X-Chat 2\ on win32 */
+	perl_auto_load_from_path (xdir);
+
+#ifdef WIN32
+	/* autoload from  C:\program files\xchat\plugins\ */
+	perl_auto_load_from_path (XCHATLIBDIR"/plugins");
+#endif
+	
+}
+
 #include <EXTERN.h>
 #define WIN32IOP_H
 #include <perl.h>
@@ -96,7 +113,7 @@ typedef struct
 	SV *callback;
 	SV *userdata;
 	xchat_hook *hook;				  /* required for timers */
-
+	unsigned int depth;
 } HookData;
 
 static PerlInterpreter *my_perl = NULL;
@@ -238,6 +255,9 @@ server_cb (char *word[], char *word_eol[], void *userdata)
 	ENTER;
 	SAVETMPS;
 
+	if (data->depth)
+		return XCHAT_EAT_NONE;
+
 	wd = newAV ();
 	sv_2mortal ((SV *) wd);
 	wd_eol = newAV ();
@@ -262,7 +282,9 @@ server_cb (char *word[], char *word_eol[], void *userdata)
 	XPUSHs (data->userdata);
 	PUTBACK;
 
+	data->depth++;
 	count = call_sv (data->callback, G_EVAL);
+	data->depth--;
 	SPAGAIN;
 	if (SvTRUE (ERRSV)) {
 		xchat_printf (ph, "Error in server callback %s", SvPV_nolen (ERRSV));
@@ -300,6 +322,9 @@ command_cb (char *word[], char *word_eol[], void *userdata)
 	ENTER;
 	SAVETMPS;
 
+	if (data->depth)
+		return XCHAT_EAT_NONE;
+
 	wd = newAV ();
 	sv_2mortal ((SV *) wd);
 	wd_eol = newAV ();
@@ -325,7 +350,9 @@ command_cb (char *word[], char *word_eol[], void *userdata)
 	XPUSHs (data->userdata);
 	PUTBACK;
 
+	data->depth++;
 	count = call_sv (data->callback, G_EVAL);
+	data->depth--;
 	SPAGAIN;
 	if (SvTRUE (ERRSV)) {
 		xchat_printf (ph, "Error in command callback %s", SvPV_nolen (ERRSV));
@@ -363,6 +390,9 @@ print_cb (char *word[], void *userdata)
 	ENTER;
 	SAVETMPS;
 
+	if (data->depth)
+		return XCHAT_EAT_NONE;
+
 	wd = newAV ();
 	sv_2mortal ((SV *) wd);
 
@@ -391,7 +421,9 @@ print_cb (char *word[], void *userdata)
 	XPUSHs (data->userdata);
 	PUTBACK;
 
+	data->depth++;
 	count = call_sv (data->callback, G_EVAL);
+	data->depth--;
 	SPAGAIN;
 	if (SvTRUE (ERRSV)) {
 		xchat_printf (ph, "Error in print callback %s", SvPV_nolen (ERRSV));
@@ -595,6 +627,7 @@ XS (XS_Xchat_hook_server)
 		SvREFCNT_inc (data->callback);
 		data->userdata = sv_mortalcopy (userdata);
 		SvREFCNT_inc (data->userdata);
+		data->depth = 0;
 		hook = xchat_hook_server (ph, name, pri, server_cb, data);
 
 		XSRETURN_UV (PTR2UV (hook));
@@ -635,6 +668,7 @@ XS (XS_Xchat_hook_command)
 		SvREFCNT_inc (data->callback);
 		data->userdata = sv_mortalcopy (userdata);
 		SvREFCNT_inc (data->userdata);
+		data->depth = 0;
 		hook = xchat_hook_command (ph, name, pri, command_cb, help_text, data);
 
 		XSRETURN_UV (PTR2UV (hook));
@@ -673,6 +707,7 @@ XS (XS_Xchat_hook_print)
 		SvREFCNT_inc (data->callback);
 		data->userdata = sv_mortalcopy (userdata);
 		SvREFCNT_inc (data->userdata);
+		data->depth = 0;
 		hook = xchat_hook_print (ph, name, pri, print_cb, data);
 
 		XSRETURN_UV (PTR2UV (hook));
@@ -1070,7 +1105,7 @@ perl_init (void)
 	   the xs_init function. (TheHobbit) */
 	int warn;
 	char *perl_args[] = { "", "-e", "0", "-w" };
-	const char perl_definitions[] = {
+	static const char perl_definitions[] = {
 		/* Redefine the $SIG{__WARN__} handler to have XChat
 		   printing warnings in the main window. (TheHobbit) */
 #include "xchat.pm.h"
@@ -1160,7 +1195,7 @@ perl_command_unloadall (char *word[], char *word_eol[], void *userdata)
 		return XCHAT_EAT_XCHAT;
 	}
 
-	return XCHAT_EAT_NONE;
+	return XCHAT_EAT_XCHAT;
 }
 
 static int
@@ -1172,7 +1207,7 @@ perl_command_reloadall (char *word[], char *word_eol[], void *userdata)
 
 		return XCHAT_EAT_XCHAT;
 	}
-	return XCHAT_EAT_NONE;
+	return XCHAT_EAT_XCHAT;
 }
 
 static int
@@ -1234,15 +1269,13 @@ perl_command_reload (char *word[], char *word_eol[], void *userdata)
 
 		len = strlen (file);
 
-
-		len = strlen (word_eol[2]);
 		if (len > 3 && strcasecmp (".pl", file + len - 3) == 0) {
 			execute_perl (sv_2mortal (newSVpv ("Xchat::Embed::reload", 0)),
 							  file);
 			return XCHAT_EAT_XCHAT;
 		}
 	}
-	return XCHAT_EAT_NONE;
+	return XCHAT_EAT_XCHAT;
 }
 
 void
