@@ -910,7 +910,7 @@ xchat_set_context (xchat_plugin *ph, xchat_context *context)
 xchat_context *
 xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 {
-	GSList *slist, *clist;
+	GSList *slist, *clist, *sessions = NULL;
 	server *serv;
 	session *sess;
 	char *netname;
@@ -939,7 +939,16 @@ xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 				if (sess->server == serv)
 				{
 					if (rfc_casecmp (channel, sess->channel) == 0)
-						return sess;
+					{
+						if (sess->server == ph->context->server)
+						{
+							g_slist_free (sessions);
+							return sess;
+						} else
+						{
+							sessions = g_slist_prepend (sessions, sess);
+						}
+					}
 				}
 				clist = clist->next;
 			}
@@ -947,6 +956,14 @@ xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 		slist = slist->next;
 	}
 
+	if (sessions)
+	{
+		sessions = g_slist_reverse (sessions);
+		sess = sessions->data;
+		g_slist_free (sessions);
+		return sess;
+	}
+
 	return NULL;
 }
 
@@ -1043,12 +1060,17 @@ xchat_get_prefs (xchat_plugin *ph, const char *name, const char **string, int *i
 	int i = 0;
 
 	/* some special run-time info (not really prefs, but may aswell throw it in here) */
-	if (!strcmp (name, "state_cursor"))
+	switch (str_hash (name))
 	{
-		*integer = fe_get_inputbox_cursor (ph->context);
-		return 2;
-	}
+		case 0xf82136c4: /* state_cursor */
+			*integer = fe_get_inputbox_cursor (ph->context);
+			return 2;
 
+		case 0xd1b: /* id */
+			*integer = ph->context->server->id;
+			return 2;
+	}
+	
 	do
 	{
 		if (!strcasecmp (name, vars[i].name))
@@ -1470,7 +1492,7 @@ xchat_send_modes (xchat_plugin *ph, const char **targets, int ntargets, int mode
 char *
 xchat_strip (xchat_plugin *ph, const char *str, int len, int flags)
 {
-	return strip_color ((char *)str, len, (flags & 1), (flags & 2));
+	return strip_color ((char *)str, len, flags);
 }
 
 void
