@@ -65,6 +65,9 @@ extern SSL_CTX *ctx;				  /* xchat.c */
 static struct session *g_sess = NULL;
 #endif
 
+static GSList *away_list = NULL;
+GSList *serv_list = NULL;
+
 static void auto_reconnect (server *serv, int send_quit, int err);
 static void server_disconnect (session * sess, int sendquit, int err);
 static int server_cleanup (server * serv);
@@ -143,8 +146,8 @@ tcp_send_queue (server *serv)
 	if (!is_server (serv))
 		return 0;
 
-	/* first try priority 1 entries */
-	pri = 1;
+	/* try priority 2,1,0 */
+	pri = 2;
 	while (pri >= 0)
 	{
 		list = serv->outbound_queue;
@@ -200,14 +203,26 @@ tcp_send_len (server *serv, char *buf, int len)
 		return tcp_send_real (serv, buf, len);
 
 	dbuf = malloc (len + 2);	/* first byte is the priority */
-	dbuf[0] = 1;	/* pri 1 for most things */
+	dbuf[0] = 2;	/* pri 2 for most things */
 	memcpy (dbuf + 1, buf, len);
 	dbuf[len + 1] = 0;
 
 	/* privmsg and notice get a lower priority */
 	if (strncasecmp (dbuf + 1, "PRIVMSG", 7) == 0 ||
 		 strncasecmp (dbuf + 1, "NOTICE", 6) == 0)
-		dbuf[0] = 0;
+	{
+		dbuf[0] = 1;
+	}
+	else
+	{
+		/* WHO/MODE get the lowest priority */
+		if (strncasecmp (dbuf + 1, "WHO ", 4) == 0 ||
+		/* but only MODE queries, not changes */
+			(strncasecmp (dbuf + 1, "MODE", 4) == 0 &&
+			 strchr (dbuf, '-') == NULL &&
+			 strchr (dbuf, '+') == NULL))
+			dbuf[0] = 0;
+	}
 
 	serv->outbound_queue = g_slist_append (serv->outbound_queue, dbuf);
 	serv->sendq_len += len; /* tcp_send_queue uses strlen */
@@ -228,7 +243,7 @@ void
 tcp_sendf (server *serv, char *fmt, ...)
 {
 	va_list args;
-	/* keep this buffer in BSS. Converting UTF-8 to ISO-8895-x might make the
+	/* keep this buffer in BSS. Converting UTF-8 to ISO-8859-x might make the
       string shorter, so allow alot more than 512 for now. */
 	static char send_buf[1540];	/* good code hey (no it's not overflowable) */
 	int len;
@@ -319,7 +334,7 @@ server_inline (server *serv, char *line, int len)
 				utf_line_allocated = g_convert_with_fallback (conv_line, conv_len, "UTF-8", encoding, "?", &read_len, &utf_len, &err);
 				if (err != NULL)
 				{
-					if (err->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE)
+					if (err->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE && conv_len > (read_len + 1))
 					{
 						/* Make our best bet by removing the erroneous char.
 						   This will work for casual 8-bit strings with non-standard chars. */
@@ -368,7 +383,6 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 	int sok = serv->sok;
 	int error, i, len;
 	char lbuf[2050];
-	char *temp;
 
 	while (1)
 	{
@@ -385,7 +399,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 			error = 0;
 			if (len < 0)
 			{
-				if (would_block_again ())
+				if (would_block ())
 					return TRUE;
 				error = sock_error ();
 			}
@@ -420,15 +434,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 
 			case '\n':
 				serv->linebuf[serv->pos] = 0;
-				if (prefs.stripcolor)
-				{
-					temp = strip_color (serv->linebuf, -1, 1, 1);
-					server_inline (serv, temp, strlen (temp));
-					free (temp);
-				} else
-				{
-					server_inline (serv, serv->linebuf, serv->pos);
-				}
+				server_inline (serv, serv->linebuf, serv->pos);
 				serv->pos = 0;
 				break;
 
@@ -478,7 +484,8 @@ server_connected (server * serv)
 						 NULL, NULL, 0);
 	}
 
-	set_server_name (serv, serv->servername);
+	server_set_name (serv, serv->servername);
+	fe_server_event (serv, FE_SE_CONNECT, 0);
 }
 
 #ifdef WIN32
@@ -503,6 +510,12 @@ server_stopconnecting (server * serv)
 		serv->iotag = 0;
 	}
 
+	if (serv->joindelay_tag)
+	{
+		fe_timeout_remove (serv->joindelay_tag);
+		serv->joindelay_tag = 0;
+	}
+
 #ifndef WIN32
 	/* kill the child process trying to connect */
 	kill (serv->childpid, SIGKILL);
@@ -533,6 +546,7 @@ server_stopconnecting (server * serv)
 	fe_progressbar_end (serv);
 
 	serv->connecting = FALSE;
+	fe_server_event (serv, FE_SE_DISCONNECT, 0);
 }
 
 #ifdef USE_OPENSSL
@@ -782,6 +796,7 @@ auto_reconnect (server *serv, int send_quit, int err)
 		serv->reconnect_away = serv->is_away;
 
 	serv->recondelay_tag = fe_timeout_add (del, timeout_auto_reconnect, serv);
+	fe_server_event (serv, FE_SE_RECONDELAY, del);
 }
 
 static void
@@ -975,6 +990,12 @@ server_cleanup (server * serv)
 		serv->iotag = 0;
 	}
 
+	if (serv->joindelay_tag)
+	{
+		fe_timeout_remove (serv->joindelay_tag);
+		serv->joindelay_tag = 0;
+	}
+
 #ifdef USE_OPENSSL
 	if (serv->ssl)
 	{
@@ -1024,6 +1045,8 @@ server_disconnect (session * sess, int sendquit, int err)
 		server_sendquit (sess);
 	}
 
+	fe_server_event (serv, FE_SE_DISCONNECT, 0);
+
 	/* close all sockets & io tags */
 	switch (server_cleanup (serv))
 	{
@@ -1513,6 +1536,7 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	serv->port = port;
 	serv->no_login = no_login;
 
+	fe_server_event (serv, FE_SE_CONNECTING, 0);
 	fe_set_away (serv);
 	server_flush_queue (serv);
 
@@ -1537,6 +1561,9 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 										(LPTHREAD_START_ROUTINE)server_child,
 										serv, 0, (DWORD *)&pid));
 #else
+#ifdef LOOKUPD
+	rand();	/* CL: net_resolve calls rand() when LOOKUPD is set, so prepare a different seed for each child. This method giver a bigger variation in seed values than calling srand(time(0)) in the child itself. */
+#endif
 	switch (pid = fork ())
 	{
 	case -1:
@@ -1550,8 +1577,8 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	}
 #endif
 	serv->childpid = pid;
-	serv->iotag =
-		fe_input_add (serv->childread, FIA_READ|FIA_FD, server_read_child, serv);
+	serv->iotag = fe_input_add (serv->childread, FIA_READ, server_read_child,
+										 serv);
 }
 
 void
@@ -1619,6 +1646,12 @@ server_new (void)
 	return serv;
 }
 
+int
+is_server (server *serv)
+{
+	return g_slist_find (serv_list, serv) ? 1 : 0;
+}
+
 void
 server_set_defaults (server *serv)
 {
@@ -1658,3 +1691,130 @@ server_get_network (server *serv, gboolean fallback)
 
 	return NULL;
 }
+
+void
+server_set_name (server *serv, char *name)
+{
+	GSList *list = sess_list;
+	session *sess;
+
+	if (name[0] == 0)
+		name = serv->hostname;
+
+	/* strncpy parameters must NOT overlap */
+	if (name != serv->servername)
+	{
+		safe_strcpy (serv->servername, name, sizeof (serv->servername));
+	}
+
+	while (list)
+	{
+		sess = (session *) list->data;
+		if (sess->server == serv)
+			fe_set_title (sess);
+		list = list->next;
+	}
+
+	if (serv->server_session->type == SESS_SERVER)
+	{
+		if (serv->network)
+		{
+			safe_strcpy (serv->server_session->channel, ((ircnet *)serv->network)->name, CHANLEN);
+		} else
+		{
+			safe_strcpy (serv->server_session->channel, name, CHANLEN);
+		}
+		fe_set_channel (serv->server_session);
+	}
+}
+
+struct away_msg *
+server_away_find_message (server *serv, char *nick)
+{
+	struct away_msg *away;
+	GSList *list = away_list;
+	while (list)
+	{
+		away = (struct away_msg *) list->data;
+		if (away->server == serv && !serv->p_cmp (nick, away->nick))
+			return away;
+		list = list->next;
+	}
+	return NULL;
+}
+
+static void
+server_away_free_messages (server *serv)
+{
+	GSList *list, *next;
+	struct away_msg *away;
+
+	list = away_list;
+	while (list)
+	{
+		away = list->data;
+		next = list->next;
+		if (away->server == serv)
+		{
+			away_list = g_slist_remove (away_list, away);
+			if (away->message)
+				free (away->message);
+			free (away);
+			next = away_list;
+		}
+		list = next;
+	}
+}
+
+void
+server_away_save_message (server *serv, char *nick, char *msg)
+{
+	struct away_msg *away = server_away_find_message (serv, nick);
+
+	if (away)						  /* Change message for known user */
+	{
+		if (away->message)
+			free (away->message);
+		away->message = strdup (msg);
+	} else
+		/* Create brand new entry */
+	{
+		away = malloc (sizeof (struct away_msg));
+		if (away)
+		{
+			away->server = serv;
+			safe_strcpy (away->nick, nick, sizeof (away->nick));
+			away->message = strdup (msg);
+			away_list = g_slist_prepend (away_list, away);
+		}
+	}
+}
+
+void
+server_free (server *serv)
+{
+	serv->cleanup (serv);
+
+	serv_list = g_slist_remove (serv_list, serv);
+
+	dcc_notify_kill (serv);
+	serv->flush_queue (serv);
+	server_away_free_messages (serv);
+
+	free (serv->nick_modes);
+	free (serv->nick_prefixes);
+	free (serv->chanmodes);
+	free (serv->chantypes);
+	if (serv->bad_nick_prefixes)
+		free (serv->bad_nick_prefixes);
+	if (serv->last_away_reason)
+		free (serv->last_away_reason);
+	if (serv->encoding)
+		free (serv->encoding);
+
+	fe_server_callback (serv);
+
+	free (serv);
+
+	notify_cleanup ();
+}
