@@ -63,14 +63,19 @@ static GOptionEntry entries[] = {
 	{ NULL }
 };
 
-void joind (int action, struct server *serv);
+static void setup_sm (gint argc, gchar *argv[]);
+static gint save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
+                          gint is_shutdown, GnomeInteractStyle interact_style,
+                          gint is_fast, gpointer client_data);
+static void kill_session (GnomeClient* client, gpointer client_data);
 
 int
 fe_args (int argc, char *argv[])
 {
 	GError *error = NULL;
 	GOptionContext *context;
-	GnomeProgram *program;
+
+	gui.main_window = NULL;
 
 #ifdef ENABLE_NLS
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -82,16 +87,23 @@ fe_args (int argc, char *argv[])
 
 #ifdef HAVE_LIBGNOME_214
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	program = gnome_program_init (PACKAGE, VERSION,
-	                              LIBGNOMEUI_MODULE, argc, argv,
-	                              GNOME_PARAM_GOPTION_CONTEXT, context,
-	                              GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
-	                              GNOME_PROGRAM_STANDARD_PROPERTIES,
-	                              NULL);
+	gui.program = gnome_program_init (PACKAGE, VERSION,
+	                                  LIBGNOMEUI_MODULE, argc, argv,
+	                                  GNOME_PARAM_GOPTION_CONTEXT, context,
+	                                  GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
+	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
+	                                  NULL);
 #else
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_parse (context, &argc, &argv, &error);
+	g_option_context_free (context);
+
+	gui.program = gnome_program_init (PACKAGE, VERSION,
+	                                  LIBGNOMEUI_MODULE, argc, argv,
+	                                  GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
+	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
+	                                  NULL);
 #endif
 
 	if (error) {
@@ -100,8 +112,6 @@ fe_args (int argc, char *argv[])
 		return 0;
 	}
 
-	g_option_context_free (context);
-
 	if (opt_version) {
 		g_print ("xchat-gnome %s\n", VERSION);
 		return 0;
@@ -110,6 +120,8 @@ fe_args (int argc, char *argv[])
 	if (opt_cfgdir)
 		xdir_fs = opt_cfgdir;
 
+	setup_sm (argc, argv);
+
 	return -1;
 }
 
@@ -129,12 +141,12 @@ fe_init (void)
 	gui.quit = FALSE;
 	servlist_init ();
 	palette_init ();
-	initialize_gui_1 ();
 	if (!preferences_exist ())
 		run_setup_dialog ();
 	else
 		set_version ();
 	load_preferences ();
+	initialize_gui_1 ();
 	initialize_gui_2 ();
 	run_main_window ();
 
@@ -166,6 +178,11 @@ void
 fe_main (void)
 {
 	gtk_main ();
+
+#ifdef HAVE_LIBGNOME_214
+	g_object_unref (gui.program);
+#endif
+
 	/* sleep for 3 seconds so any QUIT messages are not lost. The  */
 	/* GUI is closed at this point, so the user doesn't even know! */
 
@@ -209,13 +226,16 @@ fe_new_window (struct session *sess, int focus)
 {
 	static gboolean loaded = FALSE;
 
-	gui.current_session = sess;
-	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess);
-	topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, sess->topic);
+	if (focus) {
+		gui.current_session = sess;
+		topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, sess->topic);
+	}
+
+	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess, (gboolean) focus);
 	if (sess->type == SESS_SERVER)
 		navigation_tree_create_new_network_entry (gui.server_tree, sess);
 	else if (sess->type == SESS_CHANNEL || sess->type == SESS_DIALOG)
-		navigation_tree_create_new_channel_entry (gui.server_tree, sess);
+		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
 #ifdef USE_PLUGIN
 	if (!(opt_noplugins || loaded)) {
 		loaded = TRUE;
@@ -607,7 +627,7 @@ fe_set_throttle (server * serv)
 void
 fe_set_away (server * serv)
 {
-	/* FIXME: implement */
+	set_nickname_color (serv);
 }
 
 void
@@ -873,15 +893,53 @@ fe_userlist_set_selected (struct session *sess)
 	/* FIXME: implement? */
 }
 
-void
-joind (int action, struct server *serv)
+static void
+setup_sm (gint argc, gchar *argv[])
 {
-	switch (action) {
-	case 0:
-		/* FIXME: show join dialog(?) */
-		break;
-	case 1:
-		/* FIXME: hide join dialog(?) */
-		break;
-	};
+	GnomeClient *client;
+	const gchar *prefix;
+
+	client = gnome_master_client ();
+
+	g_signal_connect (G_OBJECT (client), "save_yourself", G_CALLBACK (save_session), argv[0]);
+	g_signal_connect (G_OBJECT (client), "die", G_CALLBACK (kill_session), NULL);
+
+	prefix = gnome_client_get_config_prefix (client);
+	/* FIXME: add discard command to remove saved state */
+}
+
+static gint
+save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
+              gint is_shutdown, GnomeInteractStyle interact_style,
+              gint is_fast, gpointer client_data)
+{
+	gchar **argv;
+	gint argc;
+	const gchar *prefix;
+
+	argv = g_new0 (gchar *, 3);
+	argv[0] = client_data;
+	argv[1] = "--no-auto";
+	argc = 2;
+
+	gnome_client_set_restart_style (client, GNOME_RESTART_IF_RUNNING);
+	gnome_client_set_restart_command (client, argc, argv);
+	gnome_client_set_clone_command (client, argc, argv);
+
+	prefix = gnome_client_get_config_prefix (client);
+	/* FIXME: save state */
+
+	g_free (argv);
+
+	return TRUE;
+}
+
+static void
+kill_session (GnomeClient* client, gpointer client_data)
+{
+	gtk_widget_hide (GTK_WIDGET (gui.main_window));
+	gtk_widget_hide (GTK_WIDGET (gui.dcc));
+	userlist_gui_hide ();
+	gui.quit = TRUE;
+	xchat_exit ();
 }
