@@ -1,7 +1,7 @@
 /*
  * irc-network-editor.h - GtkDialog subclass for editing an IrcNetwork
  *
- * Copyright (C) 2005 David Trowbridge
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,10 +19,12 @@
  *
  */
 
-#include "irc-network-editor.h"
-#include "error_dialog.h"
-#include <gtk/gtk.h>
+#include <config.h>
+#include <glib/gi18n.h>
 #include <string.h>
+#include <gtk/gtk.h>
+#include "irc-network-editor.h"
+#include "util.h"
 
 static GtkDialogClass *parent_class;
 
@@ -63,15 +65,13 @@ use_globals_set (GtkRadioButton *button, IrcNetworkEditor *e)
 	gtk_entry_set_text (GTK_ENTRY (e->realname), text);
 	g_free (text);
 
-	gtk_widget_set_sensitive (GTK_WIDGET (e->nickname), FALSE);
-	gtk_widget_set_sensitive (GTK_WIDGET (e->realname), FALSE);
+	gtk_widget_set_sensitive (e->custom_box, FALSE);
 }
 
 static void
 use_custom_set (GtkRadioButton *button, IrcNetworkEditor *e)
 {
-	gtk_widget_set_sensitive (e->nickname, TRUE);
-	gtk_widget_set_sensitive (e->realname, TRUE);
+	gtk_widget_set_sensitive (e->custom_box, TRUE);
 }
 
 static void
@@ -209,7 +209,6 @@ autojoin_edited (GtkCellRendererText *renderer, gchar *arg1, gchar *newtext, Irc
 static void
 irc_network_editor_init (IrcNetworkEditor *dialog)
 {
-	GtkSizeGroup *group;
 	gchar **enc;
 	GtkTreeSelection *server_selection, *autojoin_selection;
 	GladeXML *xml;
@@ -219,22 +218,25 @@ irc_network_editor_init (IrcNetworkEditor *dialog)
 	dialog->toplevel = NULL;
 
 	xml = NULL;
-	if (g_file_test ("irc-network-editor.glade", G_FILE_TEST_EXISTS))
-		xml = glade_xml_new ("irc-network-editor.glade", "toplevel", NULL);
+	if (g_file_test ("../../data/irc-network-editor.glade", G_FILE_TEST_EXISTS))
+		xml = glade_xml_new ("../../data/irc-network-editor.glade", "toplevel", NULL);
 	if (!xml)
 		xml = glade_xml_new (XCHATSHAREDIR "/irc-network-editor.glade", "toplevel", NULL);
 	if (!xml)
 		return;
 
 #define GW(name) ((dialog->name) = glade_xml_get_widget (xml, #name))
+	GW(network_settings_table);
+
 	GW(network_name);
 
 	GW(autoconnect);
 	GW(use_ssl);
+	GW(allow_invalid);
 	GW(cycle);
 
+	GW(nickserv_password);
 	GW(password);
-	GW(encoding_hbox);
 
 	GW(servers);
 	GW(add_server);
@@ -243,6 +245,7 @@ irc_network_editor_init (IrcNetworkEditor *dialog)
 
 	GW(use_globals);
 	GW(use_custom);
+	GW(custom_box);
 	GW(nickname);
 	GW(realname);
 
@@ -262,24 +265,18 @@ irc_network_editor_init (IrcNetworkEditor *dialog)
 	dialog->autojoin_renderer = gtk_cell_renderer_text_new ();
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->servers), GTK_TREE_MODEL (dialog->server_store));
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->servers), 0, "Server", dialog->server_renderer, "text", 0, NULL);
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->servers), 0, _("Server"), dialog->server_renderer, "text", 0, NULL);
 	g_object_set (G_OBJECT (dialog->server_renderer), "editable", TRUE, NULL);
 	dialog->server_column = gtk_tree_view_get_column (GTK_TREE_VIEW (dialog->servers), 0);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->autojoin_channels), GTK_TREE_MODEL (dialog->autojoin_store));
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->autojoin_channels), 0, "Channel", dialog->autojoin_renderer, "text", 0, NULL);
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->autojoin_channels), 0, _("Channel"), dialog->autojoin_renderer, "text", 0, NULL);
 	g_object_set (G_OBJECT (dialog->autojoin_renderer), "editable", TRUE, NULL);
 	dialog->autojoin_column = gtk_tree_view_get_column (GTK_TREE_VIEW (dialog->autojoin_channels), 0);
 
 	dialog->encoding = gtk_combo_box_new_text ();
 	gtk_widget_show (dialog->encoding);
-	gtk_box_pack_start (GTK_BOX (dialog->encoding_hbox), dialog->encoding, FALSE, TRUE, 0);
-
-	group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	gtk_size_group_add_widget (group, dialog->network_name);
-	gtk_size_group_add_widget (group, dialog->encoding);
-	gtk_size_group_add_widget (group, dialog->password);
-	g_object_unref (group);
+	gtk_table_attach_defaults (GTK_TABLE (dialog->network_settings_table), dialog->encoding, 1, 2, 5, 6);
 
 	enc = (gchar **) encodings;
 	do {
@@ -287,7 +284,6 @@ irc_network_editor_init (IrcNetworkEditor *dialog)
 		enc++;
 	} while (*enc);
 
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_APPLY,  GTK_RESPONSE_APPLY);
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK,     GTK_RESPONSE_OK);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
@@ -331,9 +327,9 @@ irc_network_editor_get_type (void)
 		};
 
 		irc_network_editor_type = g_type_register_static (GTK_TYPE_DIALOG, "IrcNetworkEditor", &irc_network_editor_info, 0);
-	}
 
-	parent_class = g_type_class_ref (GTK_TYPE_DIALOG);
+		parent_class = g_type_class_ref (GTK_TYPE_DIALOG);
+	}
 
 	return irc_network_editor_type;
 }
@@ -366,7 +362,7 @@ populate_autojoin_list (IrcNetworkEditor *e)
 	channels = g_strsplit (e->network->autojoin, ",", 0);
 	for (i = 0; channels[i]; i++) {
 		gtk_list_store_append (e->autojoin_store, &iter);
-		gtk_list_store_set (e->autojoin_store, &iter, 0, channels[i]);
+		gtk_list_store_set (e->autojoin_store, &iter, 0, channels[i], -1);
 	}
 
 	g_strfreev (channels);
@@ -379,12 +375,9 @@ irc_network_editor_populate (IrcNetworkEditor *e)
 
 	e->gconf = gconf_client_get_default ();
 
-	title = g_strdup_printf ("%s Network Properties", e->network->name);
-	gtk_window_set_title (GTK_WINDOW (e), title);
-	g_free (title);
-
 	if (!e->network->net) {
 		/* We're creating a new network. Don't populate things from the structure */
+		gtk_window_set_title (GTK_WINDOW (e), _("New Network"));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (e->use_globals), TRUE);
 		use_globals_set (GTK_RADIO_BUTTON (e->use_globals), e);
 		gtk_combo_box_set_active (GTK_COMBO_BOX (e->encoding), 0);
@@ -392,10 +385,15 @@ irc_network_editor_populate (IrcNetworkEditor *e)
 		return;
 	}
 
+	title = g_strdup_printf (_("%s Network Properties"), e->network->name);
+	gtk_window_set_title (GTK_WINDOW (e), title);
+	g_free (title);
+
 	gtk_entry_set_text (GTK_ENTRY (e->network_name), e->network->name);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (e->autoconnect),      e->network->autoconnect);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (e->use_ssl),          e->network->use_ssl);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (e->allow_invalid),    e->network->allow_invalid);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (e->cycle),            e->network->cycle);
 
 	if (e->network->use_global) {
@@ -416,6 +414,8 @@ irc_network_editor_populate (IrcNetworkEditor *e)
 
 	if (e->network->password)
 		gtk_entry_set_text (GTK_ENTRY (e->password), e->network->password);
+	if (e->network->nickserv_password)
+		gtk_entry_set_text (GTK_ENTRY (e->nickserv_password), e->network->nickserv_password);
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (e->encoding), e->network->encoding);
 
@@ -444,7 +444,7 @@ apply_changes (IrcNetworkEditor *e)
 	IrcNetwork *net;
 	GSList *s;
 	GtkTreeIter iter;
-	gchar *t1, *t2, *t3;
+	gchar *t1 = NULL, *t2, *t3;
 
 	net = e->network;
 
@@ -453,14 +453,17 @@ apply_changes (IrcNetworkEditor *e)
 	if (net->nick)     g_free (net->nick);
 	if (net->real)     g_free (net->real);
 	if (net->autojoin) g_free (net->autojoin);
+	if (net->nickserv_password) g_free (net->nickserv_password);
 
 	net->name     = g_strdup (gtk_entry_get_text (GTK_ENTRY (e->network_name)));
 	net->password = g_strdup (gtk_entry_get_text (GTK_ENTRY (e->password)));
 	net->nick     = g_strdup (gtk_entry_get_text (GTK_ENTRY (e->nickname)));
 	net->real     = g_strdup (gtk_entry_get_text (GTK_ENTRY (e->realname)));
+	net->nickserv_password = g_strdup (gtk_entry_get_text (GTK_ENTRY (e->nickserv_password)));
 
 	net->autoconnect        = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (e->autoconnect));
 	net->use_ssl            = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (e->use_ssl));
+	net->allow_invalid      = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (e->allow_invalid));
 	net->cycle              = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (e->cycle));
 	net->use_global         = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (e->use_globals));
 	net->encoding           = gtk_combo_box_get_active (GTK_COMBO_BOX (e->encoding));
@@ -470,14 +473,15 @@ apply_changes (IrcNetworkEditor *e)
 		g_free (serv->hostname);
 		g_free (serv);
 	}
-	g_slist_free (s);
-	s = NULL;
+	g_slist_free (net->servers);
+	net->servers = NULL;
 	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (e->server_store), &iter)) {
 		do {
 			char *text;
 			ircserver *serv = g_new0 (ircserver, 1);
 			gtk_tree_model_get (GTK_TREE_MODEL (e->server_store), &iter, 0, &text, -1);
 			serv->hostname = g_strdup (text);
+			net->servers = g_slist_append (net->servers, serv);
 		} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (e->server_store), &iter));
 	}
 
@@ -536,11 +540,6 @@ irc_network_editor_run (IrcNetworkEditor *editor)
 	gint response;
 
 	response = gtk_dialog_run (GTK_DIALOG (editor));
-	while (response == GTK_RESPONSE_APPLY) {
-		if (check_input (editor))
-			apply_changes (editor);
-		response = gtk_dialog_run (GTK_DIALOG (editor));
-	}
 	while (response == GTK_RESPONSE_OK) {
 		if (check_input (editor)) {
 			apply_changes (editor);
