@@ -37,7 +37,6 @@
 #include "main-window.h"
 #include "navigation-tree.h"
 #include "palette.h"
-#include "preferences-dialog.h"
 #include "preferences.h"
 #include "userlist-gui.h"
 #include "util.h"
@@ -63,7 +62,6 @@ static void on_irc_quit_activate (GtkAction *action, gpointer data);
 static void on_edit_cut_activate (GtkAction *action, gpointer data);
 static void on_edit_copy_activate (GtkAction *action, gpointer data);
 static void on_edit_paste_activate (GtkAction *action, gpointer data);
-static void on_edit_clear_activate (GtkAction *action, gpointer data);
 static void on_edit_preferences_activate (GtkAction *action, gpointer data);
 static void on_network_information_activate (GtkAction *action, gpointer data);
 static void on_network_reconnect_activate (GtkAction *action, gpointer data);
@@ -74,7 +72,6 @@ static void on_discussion_save_activate (GtkAction *action, gpointer data);
 static void on_discussion_leave_activate (GtkAction *action, gpointer data);
 static void on_discussion_close_activate (GtkAction *action, gpointer data);
 static void on_discussion_find_activate (GtkAction *action, gpointer data);
-static void on_discussion_clear_window_activate (GtkAction *action, gpointer data);
 static void on_discussion_bans_activate (GtkAction *action, gpointer data);
 static void on_discussion_topic_change_activate (GtkButton *widget, gpointer data);
 static void on_discussion_users_activate (GtkAction *action, gpointer data);
@@ -107,43 +104,41 @@ static GtkActionEntry action_entries [] = {
 	{ "PopupAction", NULL, "" },
 
 	/* IRC menu */
-	{ "IRCConnect",   GTK_STOCK_CONNECT, N_("_Connect"),        "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
-	{ "IRCDownloads", NULL,              N_("_File Transfers"), "<alt>F",     NULL, G_CALLBACK (on_irc_downloads_activate) },
-	{ "IRCQuit",      GTK_STOCK_QUIT,    N_("_Quit"),           "<control>Q", NULL, G_CALLBACK (on_irc_quit_activate) },
+	{ "IRCConnect",   NULL,           N_("_Connect"),        "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
+	{ "IRCDownloads", NULL,           N_("_File Transfers"), "<alt>F",     NULL, G_CALLBACK (on_irc_downloads_activate) },
+	{ "IRCQuit",      GTK_STOCK_QUIT, N_("_Quit"),           "<control>Q", NULL, G_CALLBACK (on_irc_quit_activate) },
 
 	/* Edit menu */
 	{ "EditCut",         GTK_STOCK_CUT,         N_("Cu_t"),         "<control>X", NULL, G_CALLBACK (on_edit_cut_activate) },
 	{ "EditCopy",        GTK_STOCK_COPY,        N_("_Copy"),        "<control>C", NULL, G_CALLBACK (on_edit_copy_activate) },
 	{ "EditPaste",       GTK_STOCK_PASTE,       N_("_Paste"),       "<control>V", NULL, G_CALLBACK (on_edit_paste_activate) },
-	{ "EditClear",       GTK_STOCK_CLEAR,       N_("C_lear"),       "",           NULL, G_CALLBACK (on_edit_clear_activate) },
 	{ "EditPreferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), "",           NULL, G_CALLBACK (on_edit_preferences_activate) },
 
 	/* Network menu */
 	{ "NetworkInformation", GTK_STOCK_DIALOG_INFO, N_("_Information"), "",                  NULL, G_CALLBACK (on_network_information_activate) },
 	{ "NetworkReconnect",   GTK_STOCK_REFRESH,     N_("_Reconnect"),   "<control>R",        NULL, G_CALLBACK (on_network_reconnect_activate) },
-	{ "NetworkDisconnect",  GTK_STOCK_STOP,        N_("_Disconnect"),  "",                  NULL, G_CALLBACK (on_network_disconnect_activate) },
+	{ "NetworkDisconnect",  GTK_STOCK_DISCONNECT,  N_("_Disconnect"),  "",                  NULL, G_CALLBACK (on_network_disconnect_activate) },
 	{ "NetworkClose",       GTK_STOCK_CLOSE,       N_("_Close"),       "<shift><control>W", NULL, G_CALLBACK (on_network_close_activate) },
-	{ "NetworkChannels",    GTK_STOCK_INDEX,       N_("_Channels..."), "<alt>C",            NULL, G_CALLBACK (on_network_channels_activate) },
+	{ "NetworkChannels",    NULL,                  N_("_Channels..."), "<alt>C",            NULL, G_CALLBACK (on_network_channels_activate) },
 
 	/* Discussion menu */
 	{ "DiscussionSave",        GTK_STOCK_SAVE,           N_("_Save Transcript"), "<control>S", NULL, G_CALLBACK (on_discussion_save_activate) },
 	{ "DiscussionLeave",       GTK_STOCK_QUIT,           N_("_Leave"),           "",           NULL, G_CALLBACK (on_discussion_leave_activate) },
 	{ "DiscussionClose",       GTK_STOCK_CLOSE,          N_("Cl_ose"),           "<control>W", NULL, G_CALLBACK (on_discussion_close_activate) },
 	{ "DiscussionFind",        GTK_STOCK_FIND,           N_("_Find"),            "<control>F", NULL, G_CALLBACK (on_discussion_find_activate) },
-	{ "DiscussionClearWindow", GTK_STOCK_CLEAR,          N_("_Clear Window"),    "<control>L", NULL, G_CALLBACK (on_discussion_clear_window_activate) },
-	{ "DiscussionChangeTopic", GTK_STOCK_REFRESH,        N_("Change _Topic"),    "<alt>T",     NULL, G_CALLBACK (on_discussion_topic_change_activate) },
+	{ "DiscussionChangeTopic", NULL,                     N_("Change _Topic"),    "<alt>T",     NULL, G_CALLBACK (on_discussion_topic_change_activate) },
 	{ "DiscussionBans",        GTK_STOCK_DIALOG_WARNING, N_("_Bans..."),         "<alt>B",     NULL, G_CALLBACK (on_discussion_bans_activate) },
 	{ "DiscussionUsers",       NULL,                     N_("_Users"),           "<control>U", NULL, G_CALLBACK (on_discussion_users_activate) },
 
 	/* Go menu */
-	{ "GoPreviousNetwork",    GTK_STOCK_GOTO_FIRST, N_("Pre_vious Network"),    "<control>Up",   NULL, G_CALLBACK (on_go_previous_network_activate) },
-	{ "GoNextNetwork",        GTK_STOCK_GOTO_LAST,  N_("Nex_t Network"),        "<control>Down", NULL, G_CALLBACK (on_go_next_network_activate) },
-	{ "GoPreviousDiscussion", GTK_STOCK_GO_BACK,    N_("_Previous Discussion"), "<alt>Up",       NULL, G_CALLBACK (on_go_previous_discussion_activate) },
-	{ "GoNextDiscussion",     GTK_STOCK_GO_FORWARD, N_("_Next Discussion"),     "<alt>Down",     NULL, G_CALLBACK (on_go_next_discussion_activate) },
+	{ "GoPreviousNetwork",    NULL, N_("Pre_vious Network"),    "<control>Up",   NULL, G_CALLBACK (on_go_previous_network_activate) },
+	{ "GoNextNetwork",        NULL, N_("Nex_t Network"),        "<control>Down", NULL, G_CALLBACK (on_go_next_network_activate) },
+	{ "GoPreviousDiscussion", NULL, N_("_Previous Discussion"), "<alt>Up",       NULL, G_CALLBACK (on_go_previous_discussion_activate) },
+	{ "GoNextDiscussion",     NULL, N_("_Next Discussion"),     "<alt>Down",     NULL, G_CALLBACK (on_go_next_discussion_activate) },
 
 	/* Help menu */
-	{ "HelpContents", GTK_STOCK_HELP,  N_("_Contents"), "",   NULL, G_CALLBACK (on_help_contents_activate) },
-	{ "HelpAbout",    GTK_STOCK_ABOUT, N_("_About"),    "",   NULL, G_CALLBACK (on_help_about_activate) },
+	{ "HelpContents", GTK_STOCK_HELP,  N_("_Contents"), "", NULL, G_CALLBACK (on_help_contents_activate) },
+	{ "HelpAbout",    GTK_STOCK_ABOUT, N_("_About"),    "", NULL, G_CALLBACK (on_help_about_activate) },
 };
 
 void
@@ -299,6 +294,7 @@ run_main_window ()
 	GtkWidget *pane, *widget;
 	GConfClient *client;
 	int width, height;
+	int x, y;
 	int h;
 
 	client = gconf_client_get_default ();
@@ -308,6 +304,10 @@ run_main_window ()
 		gtk_window_set_default_size (GTK_WINDOW (gui.main_window), 800, 550);
 	else
 		gtk_window_set_default_size (GTK_WINDOW (gui.main_window), width, height);
+	x = gconf_client_get_int (client, "/apps/xchat/main_window/x", NULL);
+	y = gconf_client_get_int (client, "/apps/xchat/main_window/y", NULL);
+	if (!(x == 0 || y == 0))
+		gtk_window_move (GTK_WINDOW (gui.main_window), x, y);
 	h = gconf_client_get_int (client, "/apps/xchat/main_window/hpane", NULL);
 	if(h != 0) {
 		GtkWidget *hpane = glade_xml_get_widget (gui.xml, "HPane");
@@ -325,6 +325,8 @@ run_main_window ()
 	gtk_widget_set_sensitive (widget, FALSE);
 	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionBansItem");
 	gtk_widget_set_sensitive (widget, FALSE);
+	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/ChannelPopup/ChannelBans");
+	gtk_widget_set_sensitive (widget, FALSE);
 }
 
 void
@@ -333,7 +335,10 @@ rename_main_window (gchar *server, gchar *channel)
 	gchar *new_title;
 
 	if (server == NULL) {
-		gtk_window_set_title (GTK_WINDOW (gui.main_window), channel);
+		if (channel && strlen (channel) != 0)
+			gtk_window_set_title (GTK_WINDOW (gui.main_window), channel);
+		else
+			gtk_window_set_title (GTK_WINDOW (gui.main_window), "XChat-GNOME");
 		return;
 	}
 	new_title = g_strconcat (server, ": ", channel, NULL);
@@ -354,10 +359,25 @@ on_irc_connect_activate (GtkAction *action, gpointer data)
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 }
 
+void
+save_main_window ()
+{
+	gint x, y;
+	GConfClient *client;
+
+	gtk_window_get_position (GTK_WINDOW (gui.main_window), &x, &y);
+	client = gconf_client_get_default ();
+	gconf_client_set_int (client, "/apps/xchat/main_window/x", x, NULL);
+	gconf_client_set_int (client, "/apps/xchat/main_window/y", y, NULL);
+	g_object_unref (client);
+}
+
 static void
 on_main_window_close (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
+	save_main_window ();
 	gui.quit = TRUE;
+
 	gtk_widget_hide (GTK_WIDGET (gui.dcc));
 	userlist_gui_hide ();
 	xchat_exit ();
@@ -366,6 +386,7 @@ on_main_window_close (GtkWidget *widget, GdkEvent *event, gpointer data)
 static void
 on_irc_quit_activate (GtkAction *action, gpointer data)
 {
+	save_main_window ();
 	gtk_widget_hide (GTK_WIDGET (gui.main_window));
 	gtk_widget_hide (GTK_WIDGET (gui.dcc));
 	userlist_gui_hide ();
@@ -376,35 +397,31 @@ on_irc_quit_activate (GtkAction *action, gpointer data)
 static void
 on_edit_cut_activate (GtkAction *action, gpointer data)
 {
-	gtk_editable_cut_clipboard(GTK_EDITABLE(gui.text_entry));
+	gtk_editable_cut_clipboard (GTK_EDITABLE (gui.text_entry));
 }
 
 static void
 on_edit_copy_activate (GtkAction *action, gpointer data)
 {
-	gtk_editable_copy_clipboard(GTK_EDITABLE(gui.text_entry));
+	gtk_editable_copy_clipboard (GTK_EDITABLE (gui.text_entry));
 }
 
 static void
 on_edit_paste_activate (GtkAction *action, gpointer data)
 {
-	gtk_editable_paste_clipboard(GTK_EDITABLE(gui.text_entry));
-}
-
-static void
-on_edit_clear_activate (GtkAction *action, gpointer data)
-{
-	gtk_editable_delete_selection(GTK_EDITABLE(gui.text_entry));
+	gtk_editable_paste_clipboard (GTK_EDITABLE (gui.text_entry));
 }
 
 static void
 on_edit_preferences_activate (GtkAction *action, gpointer data)
 {
-	PreferencesDialog *dialog;
+	if (!gui.prefs_dialog) {
+		gui.prefs_dialog = preferences_dialog_new ();
+		g_object_add_weak_pointer (G_OBJECT (gui.prefs_dialog),
+					   (gpointer *) (&gui.prefs_dialog));
+	}
 
-	dialog = preferences_dialog_new ();
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (GTK_WIDGET (dialog));
+	preferences_dialog_show (gui.prefs_dialog);
 }
 
 static void
@@ -476,7 +493,7 @@ static void
 on_discussion_leave_activate (GtkAction *action, gpointer data)
 {
 	session *s = gui.current_session;
-	if (s->type == SESS_CHANNEL) {
+	if ((s->type == SESS_CHANNEL) && (s->channel[0] != '\0')) {
 		gchar *text;
 		GConfClient *client;
 
@@ -497,18 +514,6 @@ on_discussion_close_activate (GtkAction *action, gpointer data)
 	if (s == NULL)
 		return;
 /*	navigation_tree_select_next_channel (gui.server_tree, TRUE);*/
-	if (s->type == SESS_CHANNEL) {
-		gchar *text;
-		GConfClient *client;
-
-		client = gconf_client_get_default ();
-		text = gconf_client_get_string (client, "/apps/xchat/irc/partmsg", NULL);
-		if (text == NULL)
-			text = g_strdup (_("Ex-Chat"));
-		s->server->p_part (s->server, s->channel, text);
-		g_object_unref (client);
-		g_free (text);
-	}
 	fe_close_window (s);
 	conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), s);
 	topic_label_remove_session        (TOPIC_LABEL        (gui.topic_label),        s);
@@ -521,12 +526,6 @@ on_discussion_find_activate (GtkAction *action, gpointer data)
 	find_bar_open (FIND_BAR (gui.find_bar));
 }
 
-static void
-on_discussion_clear_window_activate (GtkAction *action, gpointer data)
-{
-	conversation_panel_clear (CONVERSATION_PANEL (gui.conversation_panel), gui.current_session);
-}
-
 static void
 on_discussion_bans_activate (GtkAction *action, gpointer data)
 {
@@ -580,7 +579,7 @@ on_help_contents_activate (GtkAction *action, gpointer data)
 {
 	GError *error = NULL;
 
-	gnome_help_display_with_doc_id (NULL, NULL, "xchat-gnome.xml", NULL, &error);
+	gnome_help_display_with_doc_id (NULL, "xchat-gnome", "xchat-gnome.xml", NULL, &error);
 	if (error) {
 		error_dialog (_("Error showing help"), error->message);
 		g_error_free (error);
@@ -593,36 +592,12 @@ on_help_about_activate (GtkAction *action, gpointer data)
 	show_about_dialog ();
 }
 
-static void
-nickname_get_str_response (GtkDialog *dialog, gint arg1, gpointer entry)
-{
-	gchar *text, *buf;
-	GtkWidget *check;
-
-	if (arg1 == GTK_RESPONSE_ACCEPT || arg1 == GTK_RESPONSE_OK) {
-		text = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
-		check = glade_xml_get_widget (gui.xml, "nickname dialog check");
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)))
-			buf = g_strdup_printf ("allserv nick %s", text);
-		else
-			buf = g_strdup_printf ("nick %s", text);
-		handle_command (current_sess, buf, FALSE);
-		g_free (buf);
-	}
-	gtk_widget_hide_all (GTK_WIDGET (dialog));
-}
-
-static void
-nickname_str_enter (GtkWidget *entry, GtkWidget *dialog)
-{
-	gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
-}
-
 static void
 on_nickname_clicked (GtkButton *widget, gpointer user_data)
 {
 	GtkWidget *dialog;
-	GtkWidget *entry;
+	GtkWidget *entry, *away;
+	gint result;
 
 	if (gui.current_session == NULL)
 		return;
@@ -630,13 +605,38 @@ on_nickname_clicked (GtkButton *widget, gpointer user_data)
 
 	dialog = glade_xml_get_widget (gui.xml, "nickname dialog");
 	entry = glade_xml_get_widget (gui.xml, "nickname dialog entry");
+	away = glade_xml_get_widget (gui.xml, "nickname dialog away");
 
 	gtk_entry_set_text (GTK_ENTRY (entry), current_sess->server->nick);
-	g_signal_connect (G_OBJECT (entry), "activate",
-	                  G_CALLBACK (nickname_str_enter), dialog);
-	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (nickname_get_str_response), entry);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (away), current_sess->server->is_away);
 
-	gtk_widget_show_all (dialog);
+	result = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (result == GTK_RESPONSE_OK) {
+		GtkWidget *check_all;
+		gboolean all;
+		gchar *text, *buf;
+
+		check_all = glade_xml_get_widget (gui.xml, "nickname dialog all");
+		all = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check_all));
+
+		/* Nick */
+		text = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
+		if (all)
+			buf = g_strdup_printf ("allserv nick %s", text);
+		else
+			buf = g_strdup_printf ("nick %s", text);
+		handle_command (current_sess, buf, FALSE);
+		g_free (buf);
+
+		/* Away */
+		if (current_sess->server->is_away != gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (away))) {
+			if (all)
+				handle_command (current_sess, "allserv away", FALSE);
+			else
+				handle_command (current_sess, "away", FALSE);	
+		}
+	}
+	gtk_widget_hide (dialog);	
 }
 
 static gboolean
@@ -686,12 +686,49 @@ on_users_toggled (GtkToggleButton *widget, gpointer user_data)
 void
 set_nickname (struct server *serv, char *newnick)
 {
+	if (gui.current_session == NULL)
+		return;
+
 	if (serv == gui.current_session->server) {
 		GtkWidget *nick = glade_xml_get_widget (gui.xml, "nickname");
 		if (newnick == NULL)
 			gtk_button_set_label (GTK_BUTTON (nick), serv->nick);
 		else
 			gtk_button_set_label (GTK_BUTTON (nick), newnick);
+		set_nickname_color (serv);
+	}
+}
+
+void
+set_nickname_color (struct server *serv)
+{
+	if (gui.current_session == NULL)
+		return;
+
+	if (serv == gui.current_session->server) {
+		GtkWidget *nick_button;
+		GtkLabel *label;
+		PangoAttribute *attr;
+		PangoAttrList *l;
+		GdkColor *color;
+
+		l = pango_attr_list_new ();
+		nick_button = glade_xml_get_widget (gui.xml, "nickname");
+		label = GTK_LABEL (GTK_BIN (nick_button)->child);
+
+		if (serv->is_away) {
+			color = &(nick_button->style->fg[GTK_STATE_INSENSITIVE]);
+		} else {
+			color = &(nick_button->style->fg[GTK_STATE_NORMAL]);
+		}
+		attr = pango_attr_foreground_new (color->red, color->green, color->blue);
+
+		attr->start_index = 0;
+		attr->end_index = G_MAXUINT;
+		pango_attr_list_insert (l, attr);
+		gtk_label_set_attributes (label, l);
+
+		pango_attr_list_unref (l);
 	}
 }
 
