@@ -1,7 +1,7 @@
 /*
  * preferences-page-networks.c - helpers for the servers preferences page
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,8 @@
  *
  */
 
+#include <config.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
 #include "preferences-page-networks.h"
 #include "preferences-dialog.h"
@@ -161,9 +163,12 @@ preferences_page_networks_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_widget_set_sensitive (page->network_edit, FALSE);
 	gtk_widget_set_sensitive (page->network_remove, FALSE);
 
-	page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/servers.png", NULL);
+	if (g_file_test ("../../data/servers.png", G_FILE_TEST_EXISTS))
+		page->icon = gdk_pixbuf_new_from_file ("../../data/servers.png", NULL);
+	else
+		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/servers.png", NULL);
 	gtk_list_store_append (p->page_store, &iter);
-	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, "Networks", 2, 4, -1);
+	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("Networks"), 2, 4, -1);
 
 	page->network_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
 	page->sort_model = GTK_TREE_MODEL_SORT (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (page->network_store)));
@@ -171,7 +176,7 @@ preferences_page_networks_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_tree_view_set_model (GTK_TREE_VIEW (page->network_list), GTK_TREE_MODEL (page->sort_model));
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes ("Name", renderer, "text", 0, NULL);
+	column = gtk_tree_view_column_new_with_attributes (_("Name"), renderer, "text", 0, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (page->network_list), column);
 
 	g_signal_connect (G_OBJECT (page->network_list),   "row-activated", G_CALLBACK (edit_activated), page);
@@ -191,7 +196,7 @@ preferences_page_networks_new (gpointer prefs_dialog, GladeXML *xml)
 void
 preferences_page_networks_free (PreferencesNetworksPage *page)
 {
-	gdk_pixbuf_unref (page->icon);
+	g_object_unref (page->icon);
 	g_object_unref (page->sort_model);
 	g_object_unref (page->network_store);
 	g_free (page);
